describe("languages component test", function() {
  var component;
  var sandbox;
  var languagesService;
  var updateLanguagesBindStub = function() {};
  var updateMessageBindStub = function() {};
  var updateErrorBindStub = function() {};
  var observable = {subscribe: function() {} };
  var sortPipe = {transform: function(data) { return data; }};
  
  beforeEach(function() {
    languagesService = {
      get: function() {},
      add: function() {},
      delete: function() {}
    };   
  
    component = new app.LanguagesComponent(languagesService, sortPipe);
  
    sandbox = sinon.sandbox.create();    
  
    sandbox.stub(component.updateLanguages, 'bind')
           .withArgs(component)
           .returns(updateLanguagesBindStub);
      
    sandbox.stub(component.updateError, 'bind')
           .withArgs(component)
           .returns(updateErrorBindStub);

    sandbox.stub(component.updateMessage, 'bind')
           .withArgs(component)
           .returns(updateMessageBindStub);
      
    sandbox.stub(languagesService, 'get')
           .withArgs()
           .returns(observable);
  });                                
  
  afterEach(function() {
    sandbox.restore();
  });
  
  it('should set proper selector', function() {
    var annotations = 
      Reflect.getMetadata("annotations", app.LanguagesComponent)[0];
      
    expect(annotations.selector).to.be.eql('languages-list');
  });
  
  it('should set templateURL to languages.html', function() {
    var annotations = 
      Reflect.getMetadata("annotations", app.LanguagesComponent)[0];
      
    expect(annotations.templateUrl).to.be.eql('languages.html');
  });
  
  it('languages should be empty on create', function() {
    expect(component.languages).to.be.eql([]);
  });                                     
  
  it('message should be empty on create', function() {
    expect(component.message).to.be.eql('');
  });                                       
  
  it('getLanguages should call service and register handlers', function() {
    var observableMock = sandbox.mock(observable)
                                .expects('subscribe')
                                .withArgs(
                                  updateLanguagesBindStub,
                                  updateErrorBindStub);
  
    component.getLanguages();
    observableMock.verify();
  });                       
  
  it('updateLanguages should update languages', function() {
    var languagesStub = [{name: 'sample'}];
    
    component.updateLanguages(languagesStub);
    
    expect(component.languages).to.be.eql(languagesStub);
  });
  
  it('updateError should update message', function() {
    var error = 'some message';
    component.updateError(error);
    expect(component.message).to.be.eql(error);
  });                                          
  
  it('updateLanguages should call transform', function() {
    var fakeInput = '...some fake input...';
    var fakeOutput = '...some fake output...';
    
    sandbox.stub(sortPipe, 'transform')
           .withArgs(fakeInput)
           .returns(fakeOutput);
    
    component.updateLanguages(fakeInput);
    
    expect(component.languages).to.be.eql(fakeOutput);
  });
  
  it('should have proper providers', function() {
    var annotations = 
      Reflect.getMetadata("annotations", app.LanguagesComponent)[0];
      
    var providers = annotations.providers;
      
    expect(providers).to.be.eql([
        ng.http.HTTP_PROVIDERS,
        app.LanguagesService,
        app.LanguagesSortPipe
      ]);
  });
  
  it('should inject dependencies', function() {
    var parameters = Reflect.getMetadata('parameters', app.LanguagesComponent);
                     
    expect(parameters[0]).to.be.eql([app.LanguagesService]);
    expect(parameters[1]).to.be.eql([app.LanguagesSortPipe]);
  });
  
  
  it('ngOnInit should call getLanguages', function() {
    var getLanguagesMock = sandbox.mock(component)
                              .expects('getLanguages');
  
    component.ngOnInit();
    
    getLanguagesMock.verify();
  });

  it('should initialize newLanguage', function() {
    expect(component.newLanguage).to.be.eql({name: '', author: ''});
  });
  
  it('addLanguages should call service and register handlers', function() {
  
    var fakeLanguage = {name: 'l1', author: 'a1'};
    
    sandbox.stub(languagesService, 'add')
           .withArgs(fakeLanguage)
           .returns(observable);
  
    var observableMock = sandbox.mock(observable)
                                .expects('subscribe')
                                .withArgs(
                                  updateMessageBindStub,
                                  updateErrorBindStub);
                         
    component.newLanguage = fakeLanguage;
    component.addLanguage();
    observableMock.verify();
  });                       
  
  it('updateMessage should update message and call getLanguages', function() {
    var message = '...message...';   

    var getLanguagesMock = sandbox.mock(component)
                                  .expects('getLanguages');
                                  
    component.updateMessage(message);
    
    expect(component.message).to.be.eql(message);
    getLanguagesMock.verify();
  });
  
  it('deleteLanguage should call service and register handlers', function() {
  
    var languageId = '1234';
    
    sandbox.stub(languagesService, 'delete')
           .withArgs(languageId)
           .returns(observable);
  
    var observableMock = sandbox.mock(observable)
                                .expects('subscribe')
                                .withArgs(
                                  updateMessageBindStub,
                                  updateErrorBindStub);
                         
    component.deleteLanguage(languageId);
    observableMock.verify();
  });                       
});
